/*  Custom ProTrinket 5V Mouse Program v2
 *  by Sinclair Gurny
 *  Aug 17
 */

#include <Wire.h>
#include "Adafruit_DRV2605.h"
#include <ProTrinketHidCombo.h>

#define JOY_MODE 1
#define GYRO_MODE 1
#define MIDCL_MODE 1
#define SCRL_MODE 1
#define PG_MODE 1
#define FUN_BTNS 1
#define HAPTIC_MODE 1

// Haptic Vibration Motor
Adafruit_DRV2605 drv;
// Input devices
const int JOY_X = A6;
const int JOY_Y = A7;
const int GYRO_X = A2;
const int GYRO_Y = A3;
// Mouse buttons
const int LEFT_CLICK = A0;
const int RIGHT_CLICK = A1;
const int MID_CLICK = 12;
const int SCROLL_MODE = 9;
const int PAGE_MODE = 11;
// Other buttons
const int WIN_BTN = 5;
const int CTRL_BTN = 8;
const int ALT_BTN = 6;
const int TAB_BTN = 4;
// Extra pin
const int EXTRA = 3;

// Variables
// Mouse vars
int range = 32;
int threshold = 2;
int center = range / 2;
// Gyro vars
long gesture_timer = 0;
int last_gesture = -1;
// Other vars
uint8_t keys_pressed = 0;
uint8_t mod_key = 0;
uint8_t keys[10];
int keys_idx = 0;

// =============================================
// ============  SETUP  ========================
// =============================================

void setup() {
  // Setup pins
#if GYRO_MODE
  pinMode( GYRO_X, INPUT );
  pinMode( GYRO_Y, INPUT );
#endif
  pinMode( LEFT_CLICK,  INPUT_PULLUP );
  pinMode( RIGHT_CLICK, INPUT_PULLUP );
#if MIDCL_MODE
  pinMode( MID_CLICK,   INPUT_PULLUP );
#endif
#if SCRL_MODE
  pinMode( SCROLL_MODE, INPUT_PULLUP );
#endif
#if PG_MODE
  pinMode( PAGE_MODE,   INPUT_PULLUP );
#endif
#if FUN_BTNS
  pinMode( CTRL_BTN, INPUT_PULLUP );
  pinMode( ALT_BTN, INPUT_PULLUP );
  pinMode( WIN_BTN, INPUT_PULLUP );
  pinMode( TAB_BTN, INPUT_PULLUP );
#endif
  // Init Haptic Motor
  drv.begin();
  drv.selectLibrary(1);
  drv.setMode(DRV2605_MODE_INTTRIG);

  // set start timer for gestures
  gesture_timer = millis();

  // Start HID library
  TrinketHidCombo.begin();
}

// ====================================================
// ============  LOOP  ================================
// ====================================================

void loop() {
  // Reset keys
  keys_pressed = mod_key = keys_idx = 0;
  for ( int i = 0; i < 10; ++i ) {
    keys[i] = 0;
  }
  
  // ===== Joystick Mouse ======
  // Read values
  int x_val = readJoyAxis( JOY_Y );
  int y_val = readJoyAxis( JOY_X );

  int left_val = digitalRead( LEFT_CLICK );
  int right_val = digitalRead( RIGHT_CLICK );
  int mid_val = midClick();
  int page_val = pageBtn();
  int scroll_val = scrollBtn();

  int mouse_mask = 0;
  if ( left_val == LOW ) {
    mouse_mask |= MOUSEBTN_LEFT_MASK;
    // LEFT CLICK PATTERN
  } if ( right_val == LOW ) {
    mouse_mask |= MOUSEBTN_RIGHT_MASK;
    // RIGHT CLICK PATTERN
  } if ( mid_val == LOW ) {
    mouse_mask |= MOUSEBTN_MIDDLE_MASK;
    // MIDDLE CLICK PATTERN
  }

  if ( page_val == HIGH && scroll_val == HIGH ) { // NORMAL MOUSE MODE
    TrinketHidCombo.mouseMove( x_val, y_val, mouse_mask );
  } else if ( scroll_val == LOW ) { // ARROW KEY MODE
    if ( y_val <= -14 && x_val == 0 ) {
      // UP
      Key_Pressed( 0, KEYCODE_ARROW_UP );
    } else if ( y_val >= 14 && x_val == 0 ) {
      // DOWN
      Key_Pressed( 0, KEYCODE_ARROW_DOWN );
    } else if ( y_val == 0 && x_val >= 14 ) {
      // RIGHT
      Key_Pressed( 0, KEYCODE_ARROW_RIGHT );
    } else if ( y_val == 0 && x_val <= -14 ) {
      // LEFT
      Key_Pressed( 0, KEYCODE_ARROW_LEFT );
    }
  } else if ( page_val == LOW ) { // PAGE CONTROL MODE
    if ( y_val <= -14 && x_val == 0 ) {
      // PAGEUP - UP
      Key_Pressed( 0, KEYCODE_PAGE_UP );
      vibrate( 7 ); // soft bump
    } else if ( y_val >= 14 && x_val == 0 ) {
      // PAGEDOWN - DOWN
      Key_Pressed( 0, KEYCODE_PAGE_DOWN );
      vibrate( 7 ); // soft bump
    } else if ( y_val == 0 && x_val >= 14 ) {
      // END - RIGHT
      Key_Pressed( 0, KEYCODE_END );
      vibrate( 9 ); // soft bump
    } else if ( y_val == 0 && x_val <= -14 ) {
      // HOME - LEFT
      Key_Pressed( 0, KEYCODE_HOME );
      vibrate( 9 ); // soft bump
    }
  }

  // ===== Accelerometer gestures =====
#if GYRO_MODE
  int pulseX = pulseIn(GYRO_X, HIGH);
  int pulseY = pulseIn(GYRO_Y, HIGH);

  int accX = ((pulseX / 10) - 500) * 8;
  int accY = ((pulseY / 10) - 500) * 8;

  int orient = orientation( accY, accX );

  if ( millis() - gesture_timer > 250 ) {

    if ( orient == 0 && orient != last_gesture ) {
      // TILT UP - REFRESH (F5)
      Key_Pressed( 0, KEYCODE_F5 );
      vibrate( 52 ); // strong pulsing
    } else if ( orient == 1 && orient != last_gesture ) {
      // TILT RIGHT - GO FORWARD (ALT + ->)
      Key_Pressed( KEYCODE_MOD_LEFT_ALT, KEYCODE_ARROW_RIGHT );
      vibrate( 17 ); // strong click
    } else if ( orient == 2 && orient != last_gesture ) {
      // TILT BACK - SPLAY WINDOWS (WIN + TAB)
      Key_Pressed( KEYCODE_MOD_LEFT_GUI, KEYCODE_TAB );
      vibrate( 56 ); // sharp pulsing
    } else if ( orient == 3 && orient != last_gesture ) {
      // TILT LEFT - GO BACK (ALT + <-)
      Key_Pressed( KEYCODE_MOD_LEFT_ALT, KEYCODE_ARROW_LEFT );
      vibrate( 17 ); // strong click
    }
    last_gesture = orient;
    gesture_timer = millis();
  }
#endif
  // ===== Function buttons =====
#if FUN_BTNS

   if ( digitalRead( CTRL_BTN ) == LOW ) {
    Key_Pressed( KEYCODE_MOD_LEFT_CONTROL, 0 );
    vibrate( 9 ); // soft bump
  } if ( digitalRead( ALT_BTN ) == LOW ) {
    Key_Pressed( KEYCODE_MOD_LEFT_ALT, 0 );
    vibrate( 9 ); // soft bump
  } if ( digitalRead( WIN_BTN ) == LOW ) {
    Key_Pressed( KEYCODE_MOD_LEFT_GUI, 0 );
    vibrate( 9 ); // soft bump
  } if ( digitalRead( TAB_BTN ) == LOW ) {
    Key_Pressed( 0, KEYCODE_TAB );
    vibrate( 9 ); // soft bump
  }

#endif
  // ===== Manage BTN Clicks =====
  if ( keys_pressed == 0 ) {
    TrinketHidCombo.pressKey(0,0);
  } else {
    TrinketHidCombo.pressKeys( mod_key, keys, keys_pressed );
  }
  TrinketHidCombo.poll();
}


